#!/usr/local/bin/php
<?php

/*
 * @Copyright (c) Emergent Network Solutions 2002 - 2008
 * @(#) $Id: getCDR.php,v 1.3 2008/10/16 21:59:17 bli Exp $
 * @(#) $Name: ENTICE_4_1_BUILD_13 $
 */

/*
 * This script calls the getRatedCallrecords SOAP call.
 * 
 * note: when using just a date without time then it is assumed 00:00:00 on the date specified.
 * 
 */

/**************************************************************************
 * Configuration Variables
 **************************************************************************/
/*
 * Address of the SOAP server
 */
define('SOAP_SERVER_IP', 'localhost');
/*
 * Reseller access info
 */
define('RESELLER_ID', '12');
define('RESELLER_KEY', 'changedPassword');
/*
 * File ouput information
 */
define('OUTPUT_FOLDER', './');
define('FILE_BASE_NAME', 'ratedCalls');
define('FILE_TAG_NAME', '_'.strtotime('now'));
define('FILE_EXTENSION', '.dat');

/*
 * Possible Options for service are
 * 		- entice
 * 		- billing
 * 		- billing3
 * 		- voicemail
 */
define('SERVICE', 'billing3');

/**
 * Date format 
 *	for information on what letters mean visit http://us2.php.net/date
 */
define('DATE_FORMAT', 'm/d/Y, H:i:s');


/**************************************************************************
 * Common Functions
 **************************************************************************/
/*
 * Used to encode the Reseller's password 
 */
function mkResellerKey($key) {
     $dt = gmdate("m/d/Y",time());
     $key = substr($dt,0,6).$key.substr($dt,5);
     return md5($key);
}

/**************************************************************************
 * Main Program
 **************************************************************************/
/*
 * Verify Aruments passed in, current arguments are 
 * 		- Required
 * 			+ Start Date
 * 			+ End Date
 * 		- Optional
 * 			+ Date format
 */
if ( $argc != 3 && $argc != 5)
{
	printf("\nusage:
	".$_SERVER['PHP_SELF']." START_DATE END_DATE
	\tOR
	".$_SERVER['PHP_SELF']." START_DATE START_TIME END_DATE END_TIME
	\n\n
	example:
	".$_SERVER['PHP_SELF']." 1/1/2007 12/31/2007
	\tOR
	".$_SERVER['PHP_SELF']." 1/1/2007 00:00:00 12/31/2007 23:59:59
	\n\n
	note: when using just a date without time then it is assumed 00:00:00 on the date specified.\n\n");
}
else 
{
	/*
	 * Setup the SOAP client connection
	 */
	$soap_service = new SoapClient(
		'http://' . SOAP_SERVER_IP . ':8080/EnticeWS/services/'. SERVICE .'?wsdl',
		array(
			'trace' => 1,
			'classmap' => array(
				'SoapReturn'		=> 'SoapReturn',
				'SoapReturnError'	=> 'SoapReturnError',
			),
		)
	);
	
	/*
	 * Setup variables necessary for SOAP call
	 */
	$reseller_id = RESELLER_ID;
	$reseller_key = mkResellerKey(RESELLER_KEY);
	/*
	 * Set the time period to pull data
	 */
	if ( $argc == 3 )
	{
		$start_date = strtotime($argv[1]);
		$end_date = strtotime($argv[2]);
	}
	else 
	{
		$start_date = strtotime($argv[1].' '.$argv[2]);
		$end_date = strtotime($argv[3].' '.$argv[4]);
	}
	/*
	 * Paging used if you want to get a subsection of data
	 * 		In this case we want to start at the beginning and set the page to the
	 * 		largest	possible amount thus giving ALL the data.
	 */
	$page_number = 1;
	$page_size = PHP_INT_MAX;
	/*
	 * The following control what data is returned, look in the API documentation for details
	 * since we want all data we are not using these.
	 */
	$filter = '';
	$sort_field = '';
	
	/*
	 * Validate date settings
	 */
	if ($start_date > $end_date)
	{
		printf('Cannot have start date after end date ('.$start_date.', '.$end_date.')');
		exit(1);
	}
	
	try 
	{
		/*
		 * Make SOAP call
		 */
		$returned_data =$soap_service->getRatedCallRecords($reseller_id, $reseller_key, $start_date, $end_date, $page_number, $page_size, $sort_field, $filter);
		
		/*
		 * Write Returned data out to file
		 */
		$file_handle = fopen(OUTPUT_FOLDER.FILE_BASE_NAME.FILE_TAG_NAME.FILE_EXTENSION, 'w');
		fwrite($file_handle, $returned_data);
		fclose($file_handle);
		printf("\n".OUTPUT_FOLDER.FILE_BASE_NAME.FILE_TAG_NAME.FILE_EXTENSION.' was written for dates('.date(DATE_FORMAT, $start_date).' - '.date(DATE_FORMAT, $end_date).')'."\n");
	}
	catch (Exception $e)
	{
		/*
		 * Something went wrong print it 
		 */
		print_r($e);
	}
}


?>
